
#include <stdio.h>
#include <stdlib.h>
#include <alsa/asoundlib.h>

#define s16 short

/////////////////////////////////////////////////////////////////////////////////////////

char   *snd_device_in  = "pulse";
char   *snd_device_out = "pulse";
int      periods=3;        
#define FRAMES 256

/*
char     *snd_device_in  = "hw:CARD=PCH,DEV=0";
char     *snd_device_out = "hw:CARD=PCH,DEV=0";
int      periods=2;        
#define FRAMES 1024
*/
//
#define BUF FRAMES*2
s16     rdbuf[BUF],wrbuf[BUF];  
int     err;

int configure_alsa_audio(snd_pcm_t *device)
{

int  buffersize = FRAMES;  //frames
int  tmp;
int  nchannels   = 2;
int  sample_rate = 48000;
int  bits        = 16;

snd_pcm_hw_params_t *hw_params;
snd_pcm_uframes_t   frames;

if ((err = snd_pcm_hw_params_malloc(&hw_params)) < 0) {
    fprintf (stderr, "cannot allocate parameter structure (%s)\n",snd_strerror(err));
    exit(1);
	}
if ((err = snd_pcm_hw_params_any(device, hw_params)) < 0) {
    fprintf (stderr, "cannot initialize parameter structure (%s)\n",snd_strerror(err));
    exit(1);
	}
if ((err = snd_pcm_hw_params_set_access(device, hw_params, SND_PCM_ACCESS_RW_INTERLEAVED)) < 0) {
    fprintf (stderr, "cannot set access type: %s\n",snd_strerror(err));
    exit(1);
	}
if ((err = snd_pcm_hw_params_set_format(device, hw_params, SND_PCM_FORMAT_S16_LE)) < 0) {
    fprintf (stderr, "cannot set sample format: %s\n",snd_strerror(err));
    exit(1);
	}
//desactivo posible funcionalidad resampling del driver
//buscar donde esta implementada esta funcionalidad
err = snd_pcm_hw_params_set_rate_resample(device, hw_params, 0);
if(err)
    {
    fprintf (stderr, "snd_pcm_hw_params_set_rate_resample(): %s\n",snd_strerror(err));
    exit(1);
    }
err=snd_pcm_hw_params_set_rate(device, hw_params, sample_rate, 0);
fprintf(stderr, "snd_pcm_hw_params_SET_rate(%d): %s\n",sample_rate,snd_strerror(err));
if(err<0)exit(1);
if ((err = snd_pcm_hw_params_set_channels(device, hw_params,nchannels)) < 0) {
    fprintf (stderr, "cannot set channel count: %s\n",snd_strerror(err));
    exit(1);
	}

tmp=periods;
err=snd_pcm_hw_params_set_periods_near(device, hw_params, &tmp, 0);
fprintf(stderr, "snd_pcm_hw_params_set_periods_near(%d) me_dan:%d err:%s\n",periods, tmp, snd_strerror(err));
if(tmp!=periods)exit(1);

snd_pcm_uframes_t tmp2;
tmp2=(snd_pcm_uframes_t)buffersize;
err = snd_pcm_hw_params_set_buffer_size_near(device, hw_params, &tmp2);
fprintf(stderr,"snd_pcm_hw_params_set_buffer_size_near(%d): me_dan:%d [frames] err:%s\n", 
            buffersize,(int)tmp2,snd_strerror(err));
if(err<0)exit(1);            

if ((err = snd_pcm_hw_params(device, hw_params)) < 0) {
    fprintf(stderr, "Error setting HW params: %s\n",snd_strerror(err));
    return 1;
    }

return 0;
}



snd_pcm_t *hw;
snd_pcm_t *hr;
int restarting = 1,inframes,outframes;

void SNDReset(void)
{
if ((err = snd_pcm_open(&hw, snd_device_out, SND_PCM_STREAM_PLAYBACK, 0)) < 0) {
    fprintf(stderr, "cannot open output audio device %s: %s\n", snd_device_in, snd_strerror(err));
    exit(1);}
if ((err = snd_pcm_open(&hr, snd_device_in, SND_PCM_STREAM_CAPTURE, 0)) < 0) {
    fprintf(stderr, "cannot open input audio device %s: %s\n", snd_device_out, snd_strerror(err));
    exit(1);}
fprintf(stderr,"DAC\n");configure_alsa_audio(hw);
fprintf(stderr,"ADC\n");configure_alsa_audio(hr);
}


/////////////////////////////////////////////////////////////////////////////////////////
int overrun=0,underrun=0;
void SNDRead(void)
{
static int restart=1;
while(1)
	{
    if(restart) 
		{
        restart=0;
        snd_pcm_drop(hr); 
        snd_pcm_prepare(hr);
		}
    inframes=snd_pcm_readi(hr,rdbuf,FRAMES); //bloqueante
    if(inframes==-EAGAIN)
        continue;  //porque?
    else if(inframes<0)
        {
        if(overrun<5)fprintf(stderr,"ADC buf overrun\n");
        overrun++;
        restart=1;
        continue;
    	}
    else if(inframes!=FRAMES)
        {
        fprintf(stderr, "ADC leo %d frames, esperaba %d\n", inframes, FRAMES);
        continue;
        }
    else return;    
    }
}



void SNDWrite(void)
{
static int restart=1;
while(1)
	{
    if(restart) 
		{
        restart=0;
        snd_pcm_drop(hw);
        snd_pcm_prepare(hw);
		//* fill the whole output buffer 
		int i;
		for (i=0;i<BUF;    i++)wrbuf[i]=0;
		for (i=0;i<periods;i++)snd_pcm_writei(hw,wrbuf,FRAMES);
		}
    outframes=snd_pcm_writei(hw,wrbuf,inframes);
    if (outframes == -EAGAIN)
        continue;
    else if (outframes<0)
        {
        if(underrun<5)fprintf(stderr,"DAC buf underrun\n");
        underrun++;
        restart=1;
        continue;
	    }
    else if(outframes!=FRAMES)
        {
        fprintf(stderr, "DAC escribo %d frames, esperaba %d\n", outframes, FRAMES);
        continue;
        }
    else return;    
    }
}


